function sanity_check( this, State, s )

% sanity_check( this, State, s )
%
% Note that this function does not check that all variables in the data are
% active in the current grouping!


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% duplicate active groups?
x = sort(State.Ind);
for i=1:length(x)-1
  if x(i) == x(i+1)
    fprintf([ 'sanity_check: duplicate active groups ' ...
              'detected, entering debugger..\n' ]);
    keyboard;
  end
end


% State.vars corrupted?
vars = sort([ State.Groups( State.Ind ).Vars ]);
if length(vars) ~= length(State.vars) || any(vars~=sort(State.vars)),
  fprintf([ 'sanity_check: State.vars is corrupted, ' ...
            'entering debugger..\n' ]);
  keyboard;
end


% duplicate variables active?
vars = sort( State.vars );
for i=1:length(vars)-1
  if vars(i) == vars(i+1)
    fprintf([ 'sanity_check: duplicate active variables ' ...
              'detected, entering debugger..\n' ]);
    keyboard;
  end
end


% some variables missing?
% (note that this test would trigger inside Split, so it cannot be activated
%  permanently during a normal full run)
%vars = sort( State.vars );
%[dummy, varcount] = get_data_dimension( State.modeler_class, State.data ); 
%if length(vars) ~= varcount || any(vars~=1:varcount)
%  fprintf([ 'sanity_check: some variables are missing ' ...
%            'from the active grouping, entering debugger..\n' ]);
%  keyboard;
%end


% non-precomputed active groups? (mdl_vq specific!)
if strcmp( State.modeler_class, 'mdl_vq' )
  for ind=State.Ind
    x = struct(State.Groups(ind).Model);
    if isempty( x.vq.params )
    fprintf([ 'sanity_check: non-precomputed active ' ...
              'groups detected (mdl_vq -specific), entering debugger..\n' ]);
    keyboard;
    end
  end
end


% wrong models in some active groups? (mdl_vq specific!)
if strcmp( State.modeler_class, 'mdl_vq' )
  for ind=State.Ind
    v1 = sort( State.Groups(ind).Vars );
    v2 = sort( getfield( struct(State.Groups(ind).Model), 'vars' ) );
    if length(v1) ~= length(v2) || any(v1~=v2)
      fprintf([ 'sanity_check: some active groups contain ' ...
                'models for the wrong variables (mdl_vq -specific), '...
                'entering debugger..\n' ]);
      keyboard;
    end
  end
end


% all ok
return;
