function cost = vq_cost(D, Params, Vars, implicit_noisevar)

% Compute the cost using given model parameters and data
% Author: Jeremias Seppa


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.


%if isstruct(D.data)  % 'auxiliary invocation' for 
%  X1=D.data.data1;   % e.g. debugging purposes. activate
%  X2=D.data.data2;   % by removing comment marks
%  S=D.S;
%else
 types=D.types(Vars);
 X=D.data(:,Vars);
 S=D.S(Vars);
 X1=X(:,(types==1));
 X2=X(:,(types==2));
 S=S(types==2);
%end

Uhat = Params.U_hat;
U_p = Params.U_p;
Mu_mu = Params.Mu_mu;
S2_mu = Params.S2_mu;
Mu_bar = Params.Mu_bar;
Mu_tilde = Params.Mu_tilde;
Alpha_ksi = Params.Alpha_ksi;
Beta_ksi = Params.Beta_ksi;
Ksi_alpha = Params.Ksi_alpha;
Ksi_beta = Params.Ksi_beta;
c_p = Params.C_prior;
logc = Params.logC;
F = Params.F;
W = Params.W;
Ksi_log   = (psi(Ksi_alpha)-log(Ksi_beta));
S2_x      = Ksi_beta./Ksi_alpha;

[N,dim1]=size(X1);
dim2=size(X2,2);
C=size(W,1);

Mu_mu     = repmat(Mu_mu,C,1);
S2_mu     = repmat(S2_mu,C,1);
Alpha_ksi = repmat(Alpha_ksi,C,1);
Beta_ksi  = repmat(Beta_ksi,C,1);

l_codebook = C*(gammaln(c_p)) - dim1*C/2;

for j=1:dim2,
  l_codebook=l_codebook - (C*gammaln(U_p(j)*S(j))) + ...
      C*S(j)*gammaln(U_p(j)) + ...
      sum(gammaln(sum(Uhat{j},2))) - sum(sum(gammaln(Uhat{j})));
  l_codebook=l_codebook + sum(sum((Uhat{j}-U_p(j)) .* ...
				  (psi(Uhat{j}) - ...
				   repmat(psi(sum(Uhat{j},2)),1,S(j)))));
end

if dim1
 l_codebook=l_codebook + ...
   sum(sum(((log(S2_mu./Mu_tilde)+ ...
	     (Mu_tilde+(Mu_bar-Mu_mu).^2)./S2_mu)/2)));

 l_codebook=l_codebook + ...
   sum(sum(gammaln(Alpha_ksi)-gammaln(Ksi_alpha)+Ksi_alpha.*log(Ksi_beta) ...
	   - Alpha_ksi .* log(Beta_ksi) + (Ksi_alpha - Alpha_ksi) .* ...
	   Ksi_log + (Beta_ksi - Ksi_beta) .* Ksi_alpha./Ksi_beta));
end
l_codebook=l_codebook - gammaln(C*c_p) + sum(F.*logc - gammaln(F+c_p)) +...
    gammaln((C*c_p)+sum(F));
  
l_winners=sum(sum(W.*log(W+(1e-100)))) + sum(-F .* logc);

temp=zeros(C,N);
for j=1:dim2,
  UhatPsi=psi(Uhat{j});
  temp=temp - UhatPsi(:,X2(:,j))+repmat(psi(sum(Uhat{j},2)),1,N);
end
for j=1:dim1
  temp=temp + 0.5*((repmat(Mu_tilde(:,j),1,N)+(repmat(X1(:,j)',C,1)- ...
		  repmat(Mu_bar(:,j),1,N)).^2 + implicit_noisevar)./repmat(S2_x(:,j),1,N) ...
         - repmat(Ksi_log(:,j),1,N));
end
% (or forloop over t instead of j if for is fast enough - less repmats)

l_errors = dim1*N*log(2*pi)/2 + sum(sum(W .* temp));

cost = l_codebook + l_winners + l_errors;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

