function this = set_active( this, state )

% this = set_active( this, state )
%
% this       stats object
% state      0 or 1 sets the object's state to inactive/active,
%            'revert' reverts the previous state
%
% Activates/deactivates execution stats monitoring. Ongoing operations are
% still remembered and are valid after deactivation followed by a
% reactivation.
%
% Note: all explicit state changes push the previous state into the stack,
%       so the state should always be 'reverted' once for each explicit change!


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if strcmp( state, 'revert' )

  % pop state from stack
  this.active = this.active_stack{1};
  this.active_stack = this.active_stack{2};
  return;

else
  
  % push state to stack and set new state
  this.active_stack = { this.active, this.active_stack };
  this.active = state;
  return;
  
end
